import logging

from django.core.management.base import BaseCommand
from django.utils import timezone

from davinci.gcalendar.models import GCalSync
from davinci.icalendar.decorators import handle_lock


class Command(BaseCommand):
    help = 'Sychronize Google Calendar syncs that need to be synchronized.'

    @handle_lock
    def handle(self, *args, **options):
        logger = logging.getLogger("davinci.gcalendar.gcal_sync")
        logger.debug(f"Command `gcal_sync` invoked")

        for sync in GCalSync.objects.filter(active=True):
            sync: GCalSync
            if sync.last_sync is None or timezone.now() > (sync.last_sync + sync.sync_interval):
                logger.debug(f"Synchronizing {sync.name}...")

                events = sync.get_events()
                added, existing, unchanged, deleted = sync.target.upload_events(events, purge=sync.purge, ignore_invalid_recurring=True)

                sync.last_sync = timezone.now()
                sync.save()
                logger.debug(f"Sync for {sync.name} done! {added} events added, {existing} updated, {unchanged} unchanged, and {deleted} purged.")

        logger.debug("iCal sync complete!")
