from django.conf import settings
from django.contrib import messages
from django.shortcuts import reverse, redirect
from django.contrib.auth import get_user_model
from google.oauth2 import id_token
from google_auth_oauthlib.flow import Flow
from google.auth.transport.urllib3 import Request
from urllib3 import PoolManager

from davinci.gcalendar.models import GoogleAuthInfo

User = get_user_model()

flow = Flow.from_client_secrets_file(client_secrets_file=settings.GOOGLE_CLIENT_SECRET_FILE,
                                     scopes=settings.GOOGLE_AUTH_SCOPES)


def oauth2_view(request):
    callback_url = reverse("oauth2callback")
    flow.redirect_uri = request.build_absolute_uri(callback_url).replace('http:', 'https:')
    flow.oauth2session.redirect_uri = flow.redirect_uri
    auth_url, _ = flow.authorization_url(access_type='offline', prompt='consent', include_granted_scopes='true')
    return redirect(auth_url)


def oauth2_callback_view(request):
    flow.fetch_token(authorization_response=request.build_absolute_uri().replace('http:', 'https:'))
    creds = flow.credentials
    userinfo = id_token.verify_oauth2_token(creds._id_token, Request(http=PoolManager()), creds._client_id)
    try:
        gauth = GoogleAuthInfo.objects.get(email=userinfo['email'])
        gauth.creds = creds
        gauth.save()
    except GoogleAuthInfo.DoesNotExist:
        GoogleAuthInfo.objects.create(email=userinfo['email'], creds=creds)

    # Return Response as you want or Redirect to some URL
    messages.info(request, f"Google account '{userinfo['email']}' added!")
    return redirect("admin:index")
