from typing import List
from urllib.request import urlopen

import ics
from django.utils.text import slugify
from ics import Event

from davinci.caldav.models import CalDAVCalendar


def get_ical_from_url(ical_url):
    with urlopen(ical_url) as f:
        return f.read()


def split_events(data):
    c = ics.Calendar(data.decode('utf-8'))
    return c.events


def fix_ical_uid(event: Event, padding: str):
    e = event.clone()
    e.uid = f"{event.uid}{padding}"
    return e


def fix_ical_uids(events: List[Event], calendar: CalDAVCalendar):
    # Pad the uids with some string related to the calendar
    # So that if someone wants to sync the same .ics to two calendars, the UIDs are different. We will do this by
    # appending the string "@slugified-server-name+slugified-username+slugified-calendar-name" to each uid.
    padding = f"@{slugify(calendar.server.name)}+{slugify(calendar.server.username)}+{slugify(calendar.name)}"
    return [fix_ical_uid(event, padding) for event in events]
